@@ -1,10 +1,9 @@
 from .main import Lightning
 from .session import Session
-from .visualization import Visualization
+from .visualization import Visualization, VisualizationLocal
 from .types.plots import *
-from .types.linked import *
 from .types.images import *
 from .types.streaming import *
 from .types.three import *
 
-__version__ = "1.0.6"
+__version__ = "1.1.0"
