@@ -1,32 +1,48 @@
 import requests
-import os
 from .session import Session
-from .visualization import Visualization
+from .visualization import Visualization, VisualizationLocal
 
 
 class Lightning(object):
 
-    def __init__(self, host="http://localhost:3000", ipython=False, dbcloud=False, auth=None):
-        self.set_host(host)
-        self.auth = auth
+    def __init__(self, host="http://localhost:3000", local=False, ipython=False, auth=None, size='medium'):
 
-        if auth is not None:
-            if isinstance(auth, tuple):
-                self.set_basic_auth(auth[0], auth[1])
+        if ipython:
+            self.startup_message_ipython()
+        else:
+            self.startup_message()
+
+        if local:
+            self.enable_local()
+        else:
+            self.local_enabled = False
+            self.set_host(host)
+            self.auth = auth
+
+            if auth is not None:
+                if isinstance(auth, tuple):
+                    self.set_basic_auth(auth[0], auth[1])
+
+            status = self.check_status()
+            if not status:
+                raise ValueError("Could not access server")
 
         if ipython:
             self.enable_ipython()
+            self.set_size(size)
         else:
             self.ipython_enabled = False
-
-        if dbcloud:
-            self.enable_dbcloud()
+            self.set_size('full')
 
     def __repr__(self):
+        s = 'Lightning\n'
+        if hasattr(self, 'host') and self.host is not None and not self.local_enabled:
+            s += 'host: %s\n' % self.host
+        if self.local_enabled:
+            s += 'host: local\n'
         if hasattr(self, 'session') and self.session is not None:
-            return 'Lightning server at host: %s' % self.host + '\n' + self.session.__repr__()
-        else:
-            return 'Lightning server at host: %s' % self.host
+            s += 'session: %s\n' % self.session.id
+        return s
 
     def get_ipython_markup_link(self):
         return '%s/js/ipython-comm.js' % self.host
@@ -44,16 +60,22 @@ class Lightning(object):
         # https://github.com/jakevdp/mpld3/blob/master/mpld3/_display.py#L357
 
         from IPython.core.getipython import get_ipython
-        from IPython.display import display, Javascript
+        from IPython.display import display, Javascript, HTML
 
         self.ipython_enabled = True
         ip = get_ipython()
         formatter = ip.display_formatter.formatters['text/html']
-        formatter.for_type(Visualization, lambda viz, kwds=kwargs: viz.get_html())
 
-        r = requests.get(self.get_ipython_markup_link(), auth=self.auth)
-
-        display(Javascript(r.text))
+        if self.local_enabled:
+            from lightning.visualization import VisualizationLocal
+            js = VisualizationLocal.load_embed()
+            display(HTML("<script>" + js + "</script>"))
+            print('Running local mode, some functionality limited.\n')
+            formatter.for_type(VisualizationLocal, lambda viz, kwds=kwargs: viz.get_html())
+        else:
+            formatter.for_type(Visualization, lambda viz, kwds=kwargs: viz.get_html())
+            r = requests.get(self.get_ipython_markup_link(), auth=self.auth)
+            display(Javascript(r.text))
 
     def disable_ipython(self):
         """
@@ -68,26 +90,7 @@ class Lightning(object):
         ip = get_ipython()
         formatter = ip.display_formatter.formatters['text/html']
         formatter.type_printers.pop(Visualization, None)
-
-    def enable_dbcloud(self):
-        """
-        Enable lightning in the Databricks cloud notebook.
-
-        This will automatically start a lightning server if
-        it is not already running, using the host and install
-        location expected on a Databricks cloud notebook.
-        """
-        self.host = "http://localhost:3000"
-        url = self.host + "/status/"
-        installation = '/root/lightning/'
-        try:
-            r = requests.get(url)
-            if r.status_code != 200:
-                raise Exception("Server is running but not returning 200 status")
-        except requests.ConnectionError:
-            s = os.system('node ' + installation + '/server.js &> /dev/null > /dev/null &')
-            if s != 0:
-                raise Exception("Failed to start lightning server, check path %s", installation)
+        formatter.type_printers.pop(VisualizationLocal, None)
 
     def create_session(self, name=None):
         """
@@ -109,6 +112,21 @@ class Lightning(object):
         self.session = Session(lgn=self, id=session_id)
         return self.session
 
+    def enable_local(self):
+        """
+        Enable a local mode.
+
+        Data is handled locally and embedded via templates.
+        Useful for notebooks, and can be used offline.
+        """
+        self.local_enabled = True
+
+    def disable_local(self):
+        """
+        Disable local mode
+        """
+        self.local_enabled = False
+
     def set_basic_auth(self, username, password):
         """
         Set authenatication.
@@ -131,6 +149,53 @@ class Lightning(object):
         self.host = host
         return self
 
+    def set_size(self, size='medium'):
+        """
+        Set a figure size using one of four options.
+
+        Convention is 'small': 400px, 'medium': 600px, 'large': 800px,
+        and 'full' will use the entire width
+        """
+        if size not in ['small', 'medium', 'large', 'full']:
+            raise ValueError("Size must be one of 'small', 'medium', 'large', 'full'")
+        self.size = size
+
+    def check_status(self):
+        """
+        Check the server for status
+        """
+        try:
+            r = requests.get(self.host + '/status', auth=self.auth,
+                             timeout=(10.0, 10.0))
+            if not r.status_code == requests.codes.ok:
+                print("Problem connecting to server at %s" % self.host)
+                print("status code: %s" % r.status_code)
+                return False
+            else:
+                print("Connected to server at %s" % self.host)
+                return True
+        except (requests.exceptions.ConnectionError,
+                requests.exceptions.MissingSchema,
+                requests.exceptions.InvalidSchema) as e:
+            print("Problem connecting to server at %s" % self.host)
+            print("error: %s" % e)
+            return False
+
+    def startup_message_ipython(self):
+        import os
+        import base64
+        from IPython.display import display, HTML
+        icon = os.path.join(os.path.dirname(__file__), 'lib/icon.png')
+        with open(icon, "rb") as imfile:
+            im = 'data:image/png;base64,' + base64.b64encode(imfile.read())
+        t = "<div style='margin-top:8px'><img src='%s' width='30px' height='35px' " \
+            "style='display: inline-block; padding-right: 10px'>" \
+            "</img><span>Lightning initialized</span></div>" % im
+        display(HTML(t))
+
+    def startup_message(self):
+        print("Lightning initialized")
+
     def plot(self, data=None, type=None):
         """
         Generic plotting function.
@@ -157,13 +222,6 @@ class Lightning(object):
         if not hasattr(self, 'session'):
             self.create_session()
         
-        viz = Generic.baseplot(self.session, type, data)
+        viz = Generic._baseplot(self.session, type, data)
         self.session.visualizations.append(viz)
-        return viz
-
-
-
-
-
-        
-
+        return viz
\ No newline at end of file
