@@ -1,6 +1,6 @@
 import pytest
-from numpy import random, ceil
-from lightning import Lightning, Visualization
+from numpy import random
+from lightning import Lightning, Visualization, VisualizationLocal
 
 
 class TestLightningAPIClient(object):
@@ -11,7 +11,7 @@ class TestLightningAPIClient(object):
         lgn.create_session("test-session")
         return lgn
 
-    def test_create_generic(self, lgn):
+    def test_create(self, lgn):
 
         series = random.randn(5, 100)
         viz = lgn.plot(data={"series": series}, type='line')
@@ -19,13 +19,11 @@ class TestLightningAPIClient(object):
         assert isinstance(viz, Visualization)
         assert hasattr(viz, 'id')
 
+    def test_local(self, lgn):
 
-    def test_ipython_support(self, lgn):
-
-        lgn.ipython = True
+        lgn.enable_local()
         x = random.randn(100)
         viz = lgn.line(x)
 
-        assert isinstance(viz, Visualization)
-        assert hasattr(viz, 'id')
+        assert isinstance(viz, VisualizationLocal)
 
